@@ -1,4 +1,4 @@
-#include "hydra-mod.h"
+#include "hydra-http.h"
 #include "sasl.h"
 
 extern char *HYDRA_EXIT;
@@ -8,10 +8,10 @@ char *http_buf = NULL;
 int32_t webport, freemischttp = 0;
 int32_t http_auth_mechanism = AUTH_BASIC;
 
-int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, char *miscptr, FILE * fp, char *type) {
+int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, char *miscptr, FILE * fp, char *type, ptr_header_node ptr_head) {
   char *empty = "";
-  char *login, *pass, buffer[500], buffer2[500];
-  char header[64] = "Content-Length: 0\r\n";
+  char *login, *pass, *buffer, buffer2[500];
+  char *header;
   char *ptr, *fooptr;
   int32_t complete_line = 0;
   char tmpreplybuf[1024] = "", *tmpreplybufptr;
@@ -21,8 +21,15 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
   if (strlen(pass = hydra_get_next_password()) == 0)
     pass = empty;
 
-  if (strcmp(type, "POST") != 0)
-    header[0] = 0;
+  if (strcmp(type, "POST") == 0)
+    add_header(&ptr_head, "Content-Length", "0", HEADER_TYPE_DEFAULT);
+
+  header = stringify_headers(&ptr_head);
+
+  if(!(buffer = malloc(strlen(header) + 500))) {
+    free(header);
+    return 3;
+  }
 
   // we must reset this if buf is NULL and we do MD5 digest
   if (http_buf == NULL && http_auth_mechanism == AUTH_DIGESTMD5)
@@ -36,7 +43,7 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
     sprintf(buffer2, "%.50s:%.50s", login, pass);
     hydra_tobase64((unsigned char *) buffer2, strlen(buffer2), sizeof(buffer2));
 
-    /* again: no snprintf to be portable. dont worry, buffer cant overflow */
+    /* again: no snprintf to be portable. don't worry, buffer can't overflow */
     if (use_proxy == 1 && proxy_authentication[selected_proxy] != NULL)
       sprintf(buffer, "%s http://%s:%d%.250s HTTP/1.1\r\nHost: %s\r\nConnection: close\r\nAuthorization: Basic %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s\r\n",
               type, webtarget, webport, miscptr, webtarget, buffer2, proxy_authentication[selected_proxy], header);
@@ -62,6 +69,8 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
       fooptr = buffer2;
       sasl_digest_md5(fooptr, login, pass, buffer, miscptr, type, webtarget, webport, header);
       if (fooptr == NULL) {
+	free(buffer);
+	free(header);
         return 3;
       }
 
@@ -82,7 +91,7 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
       buildAuthRequest((tSmbNtlmAuthRequest *) buf2, 0, NULL, NULL);
       to64frombits(buf1, buf2, SmbLength((tSmbNtlmAuthRequest *) buf2));
 
-      /* to be portable, no snprintf, buffer is big enough so it cant overflow */
+      /* to be portable, no snprintf, buffer is big enough so it can't overflow */
       //send the first..
       if (use_proxy == 1 && proxy_authentication[selected_proxy] != NULL)
         sprintf(buffer,
@@ -97,8 +106,11 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
                   buf1, header);
       }
 
-      if (hydra_send(s, buffer, strlen(buffer), 0) < 0)
+      if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
+	free(buffer);
+	free(header);
         return 1;
+      }
 
       //receive challenge
       if (http_buf != NULL)
@@ -109,8 +121,11 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
         http_buf = hydra_receive_line(s);
       }
 
-      if (http_buf == NULL)
+      if (http_buf == NULL) {
+	free(buffer);
+	free(header);
         return 1;
+      }
 
       if (pos != NULL) {
         char *str;
@@ -153,6 +168,8 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
   }
 
   if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
+    free(buffer);
+    free(header);
     return 1;
   }
 
@@ -189,6 +206,8 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
   if (http_buf == NULL) {
     if (verbose)
       hydra_report(stderr, "[ERROR] Server did not answer\n");
+    free(buffer);
+    free(header);
     return 3;
   }
 
@@ -228,6 +247,8 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
       if (find_auth) {
 //        free(http_buf);
 //        http_buf = NULL;
+	free(buffer);
+	free(header);
         return 1;
       }
     }
@@ -235,6 +256,8 @@ int32_t start_http(int32_t s, char *ip, int32_t port, unsigned char options, cha
   }
 //  free(http_buf);
 //  http_buf = NULL;
+  free(buffer);
+  free(header);
   if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
     return 3;
   return 1;
@@ -244,6 +267,7 @@ void service_http(char *ip, int32_t sp, unsigned char options, char *miscptr, FI
   int32_t run = 1, next_run = 1, sock = -1;
   int32_t myport = PORT_HTTP, mysslport = PORT_HTTP_SSL;
   char *ptr, *ptr2;
+  ptr_header_node ptr_head = NULL;
 
   hydra_register_socket(sp);
   if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
@@ -278,6 +302,17 @@ void service_http(char *ip, int32_t sp, unsigned char options, char *miscptr, FI
   else
     webport = mysslport;
 
+  /* Advance to options string */
+  ptr = miscptr;
+  while (*ptr != 0 && (*ptr != ':' || *(ptr - 1) == '\\'))
+    ptr++;
+  if (*ptr != 0)
+    *ptr++ = 0;
+  optional1 = ptr;
+
+  if (!parse_options(optional1, &ptr_head))
+    run = 4;
+
   while (1) {
     next_run = 0;
     switch (run) {
@@ -306,7 +341,7 @@ void service_http(char *ip, int32_t sp, unsigned char options, char *miscptr, FI
         break;
       }
     case 2:                    /* run the cracking function */
-      next_run = start_http(sock, ip, port, options, miscptr, fp, type);
+      next_run = start_http(sock, ip, port, options, miscptr, fp, type, ptr_head);
       break;
     case 3:                    /* clean exit */
       if (sock >= 0)
@@ -353,5 +388,7 @@ int32_t service_http_init(char *ip, int32_t sp, unsigned char options, char *mis
 
 void usage_http(const char* service) {
   printf("Module %s requires the page to authenticate.\n"
-         "For example:  \"/secret\" or \"http://bla.com/foo/bar\" or \"https://test.com:8080/members\"\n\n", service);
+         "The following parameters are optional:\n"
+         " (h|H)=My-Hdr\\: foo   to send a user defined HTTP header with each request\n"
+         "For example:  \"/secret\" or \"http://bla.com/foo/bar:H=Cookie\\: sessid=aaaa\" or \"https://test.com:8080/members\"\n\n", service);
 }
