@@ -283,14 +283,20 @@ int32_t internal__hydra_connect(char *host, int32_t port, int32_t protocol, int3
           snprintf(buf, 4096, "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", hydra_address2string(host), port, proxy_authentication[selected_proxy]);
 
         send(s, buf, strlen(buf), 0);
+        if (debug) {
+          char *ptr = index(buf, '\r');
+          if (ptr != NULL)
+            *ptr = 0;
+          printf("DEBUG_CONNECT_PROXY_SENT: %s\n", buf);
+        }
         recv(s, buf, 4096, 0);
         if (strncmp("HTTP/", buf, 5) == 0 && (tmpptr = index(buf, ' ')) != NULL && *++tmpptr == '2') {
           if (debug)
             printf("DEBUG_CONNECT_PROXY_OK\n");
         } else {
-          if (debug)
+          if (debug && tmpptr)
             printf("DEBUG_CONNECT_PROXY_FAILED (Code: %c%c%c)\n", *tmpptr, *(tmpptr + 1), *(tmpptr + 2));
-          if (verbose)
+          if (verbose && tmpptr)
             fprintf(stderr, "[ERROR] CONNECT call to proxy failed with code %c%c%c\n", *tmpptr, *(tmpptr + 1), *(tmpptr + 2));
           err = 1;
         }
@@ -534,7 +540,7 @@ int32_t internal__hydra_connect_to_ssl(int32_t socket, char *hostname) {
 //    SSL_CTX_set_options(sslContext, SSL_OP_NO_SSLv2);
 //    SSL_CTX_set_options(sslContext, SSL_OP_NO_TLSv1);
 
-    /* we set the default verifiers and dont care for the results */
+    /* we set the default verifiers and don't care for the results */
     (void) SSL_CTX_set_default_verify_paths(sslContext);
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
     SSL_CTX_set_tmp_rsa_callback(sslContext, ssl_temp_rsa_cb);
@@ -948,13 +954,12 @@ char *hydra_receive_line(int32_t socket) {
   } else {
     if (debug)
       printf("[DEBUG] hydra_data_ready_timed: %d, waittime: %d, conwait: %d, socket: %d\n", i, waittime, conwait, socket);
-    i = 0;
   }
 
   if (got < 0) {
     if (debug) {
       sprintf(text, "[DEBUG] RECV [pid:%d]", getpid());
-      hydra_dump_data("", -1, text);
+      hydra_dump_data((unsigned char*)"", -1, text);
       //hydra_report_debug(stderr, "DEBUG_RECV_BEGIN||END [pid:%d %d]", getpid(), i);
       perror("recv");
     }
